@@ -24,6 +24,13 @@
 #include "fanin/fanin.h"
 #include "scatterv/scatterv.h"
 
+const char *ucc_tl_ucp_alltoall_onesided_names[] = {
+    [UCC_TL_UCP_ALLTOALL_ONESIDED_PUT]   = "put",
+    [UCC_TL_UCP_ALLTOALL_ONESIDED_GET]   = "get",
+    [UCC_TL_UCP_ALLTOALL_ONESIDED_AUTO]  = "auto",
+    [UCC_TL_UCP_ALLTOALL_ONESIDED_LAST]  = NULL
+};
+
 ucc_status_t ucc_tl_ucp_get_lib_attr(const ucc_base_lib_t *lib,
                                      ucc_base_lib_attr_t  *base_attr);
 
@@ -95,6 +102,16 @@ ucc_config_field_t ucc_tl_ucp_lib_config_table[] = {
      ucc_offsetof(ucc_tl_ucp_lib_config_t, alltoallv_hybrid_chunk_byte_limit),
      UCC_CONFIG_TYPE_MEMUNITS},
 
+    {"ALLTOALL_ONESIDED_PERCENT_BW", "100",
+     "Percentage (1-100) of NIC bandwidth to use for congestion avoidance",
+     ucc_offsetof(ucc_tl_ucp_lib_config_t, alltoall_onesided_percent_bw),
+     UCC_CONFIG_TYPE_UINT},
+
+    {"ALLTOALL_ONESIDED_ALG", "auto",
+     "Algorithm to be used with onesided Alltoall",
+     ucc_offsetof(ucc_tl_ucp_lib_config_t, alltoall_onesided_alg),
+     UCC_CONFIG_TYPE_ENUM(ucc_tl_ucp_alltoall_onesided_names)},
+
     {"KN_RADIX", "0",
      "Radix of all algorithms based on knomial pattern. When set to a "
      "positive value it is used as a convenience parameter to set all "
